#include "filechooser.h"
#include <tqlineedit.h>
#include <tqpushbutton.h>
#include <tdefiledialog.h>
#include <tqlayout.h>

FileChooser::FileChooser( TQWidget *parent, const char *name )
    : TQWidget( parent, name ), md( File )
{
    TQHBoxLayout *layout = new TQHBoxLayout( this );
    layout->setMargin( 0 );

    lineEdit = new TQLineEdit( this, "filechooser_lineedit" );
    layout->addWidget( lineEdit );

    connect( lineEdit, TQT_SIGNAL( textChanged( const TQString & ) ),
	     this, TQT_SIGNAL( fileNameChanged( const TQString & ) ) );

    button = new TQPushButton( "...", this, "filechooser_button" );
    button->setFixedWidth( button->fontMetrics().width( " ... " ) );
    layout->addWidget( button );

    connect( button, TQT_SIGNAL( clicked() ),
	     this, TQT_SLOT( chooseFile() ) );

    setFocusProxy( lineEdit );
}

void FileChooser::setMode( Mode m )
{
    md = m;
}

FileChooser::Mode FileChooser::mode() const
{
    return md;
}

void FileChooser::setFileName( const TQString &fn )
{
    lineEdit->setText( fn );
}

TQString FileChooser::fileName() const
{
    return lineEdit->text();
}

void FileChooser::chooseFile()
{
    TQString fn;
    if ( mode() == File )
	fn = KFileDialog::getOpenFileName( lineEdit->text(), TQString(), this );
    else
	fn = KFileDialog::getExistingDirectory( lineEdit->text(),this );

    if ( !fn.isEmpty() ) {
	lineEdit->setText( fn );
	emit fileNameChanged( fn );
    }
}

#include "filechooser.moc"
