 /**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of TQt Designer.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "timestamp.h"
#include <tqfile.h>
#include <tqfileinfo.h>

class TQTimer;

TimeStamp::TimeStamp( TQObject *parent, const TQString &f )
    : TQObject( parent ), filename( f ), autoCheck( false )
{
    timer = new TQTimer( this );
    connect( timer, TQT_SIGNAL( timeout() ), this, TQT_SLOT( autoCheckTimeStamp() ) );
    update();
}

void TimeStamp::setFileName( const TQString &f )
{
    filename = f;
    update();
}

TQString TimeStamp::fileName() const
{
    return filename;
}

void TimeStamp::setAutoCheckEnabled( bool a )
{
    autoCheck = a;
    if ( autoCheck )
	timer->start( 5000 );
    else
	timer->stop();
}

void TimeStamp::update()
{
    TQFile f( filename );
    if ( !f.exists() )
	return;
    lastTimeStamp = TQFileInfo( f ).lastModified();
}

bool TimeStamp::isUpToDate() const
{
    TQFile f( filename );
    if ( !f.exists() )
	return true;
    return lastTimeStamp == TQFileInfo( f ).lastModified();
}

bool TimeStamp::isAutoCheckEnabled() const
{
    return autoCheck;
}

void TimeStamp::autoCheckTimeStamp()
{
    if ( !isUpToDate() )
	emit timeStampChanged();
}
#include "timestamp.moc"
