/***************************************************************************
                          dialog.h  -  Kommander dialog
                             -------------------
    copyright            : (C) 2002 by Marc Britton <consume@optushome.com.au>
                           (C) 2004 by Andras Mantia <amantia@kde.org>
                           (C) 2004 by Michal Rudolf <mrudolf@kdewebdev.org>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _HAVE_DIALOG_H_
#define _HAVE_DIALOG_H_

/* KDE INCLUDES */

/* QT INCLUDES */
#include <tqdialog.h>

/* OTHER INCLUDES */
#include <kommanderwindow.h>
#include <kommander_export.h>
class TQShowEvent;
class KOMMANDER_EXPORT Dialog : public TQDialog, public KommanderWindow
{
  Q_OBJECT
  

  TQ_PROPERTY(TQString populationText READ populationText WRITE setPopulationText DESIGNABLE false)
  TQ_PROPERTY(TQStringList associations READ associatedText WRITE setAssociatedText DESIGNABLE false)
  TQ_PROPERTY(bool KommanderWidget READ isKommanderWidget)
  TQ_PROPERTY(bool useInternalParser READ useInternalParser WRITE setUseInternalParser)
  TQ_PROPERTY(bool useShebang READ useShebang WRITE setUseShebang DESIGNABLE true)
  TQ_PROPERTY(TQString shebang READ Shebang WRITE setShebang DESIGNABLE true)
  TQ_OVERRIDE(bool modal DESIGNABLE false)
  
public:
  Dialog(TQWidget *, const char *, bool = true, int=0);
  ~Dialog();
  virtual bool isKommanderWidget() const;
  virtual bool useInternalParser() const;
  virtual void setUseInternalParser(bool b);
  virtual void setAssociatedText(const TQStringList&);
  virtual TQStringList associatedText() const;
  virtual TQString currentState() const;
  virtual TQString populationText() const;
  virtual void setPopulationText(const TQString&);
  virtual void show();

  bool useShebang() const { return m_useShebang; }
  void setUseShebang(bool useit) { m_useShebang = useit; }
  TQString Shebang() const { return m_shebang; }
  void setShebang(const TQString &text) { m_shebang = text; }
  
  virtual TQString handleDCOP(int function, const TQStringList& args);
  virtual bool isFunctionSupported(int function);
public slots:
  virtual void setWidgetText(const TQString&);
  virtual void exec();
  virtual void populate();
  virtual void done(int r);
protected slots:
  virtual void initialize();
  virtual void destroy();
  
signals:
  void widgetOpened();
  void widgetTextChanged(const TQString&);
  void finished();
  void contextMenuRequested(int xpos, int ypos);
protected:
  void showEvent( TQShowEvent *e );
  void keyPressEvent( TQKeyEvent * );
  void contextMenuEvent( TQContextMenuEvent * e );
private:
  TQString m_fileName;
  bool m_useInternalParser;
  bool m_useShebang; //add m_shebang at the beginning of the dialog code or not
  bool m_firstShow; //tells if the show method was called before or not
  TQString m_shebang; //#!/usr/bin/kmdr-executor or similar
};

#endif
